@@ -1,5 +1,6 @@
 #' Fast Imputation of Missing Values by Chained Random Forests
 #' 
+#' @description
 #' Uses the "ranger" package (Wright & Ziegler) to do fast missing value imputation by 
 #' chained random forests, see Stekhoven & Buehlmann and Van Buuren & Groothuis-Oudshoorn.
 #' Between the iterative model fitting, it offers the option of predictive mean matching. 
@@ -8,11 +9,17 @@
 #' Secondly, predictive mean matching tries to raise the variance in the resulting 
 #' conditional distributions to a realistic level. This allows to do multiple imputation 
 #' when repeating the call to [missRanger()]. 
+#' 
+#' @details
 #' The iterative chaining stops as soon as `maxiter` is reached or if the average 
-#' out-of-bag estimate of performance stops improving. 
-#' In the latter case, except for the first iteration, the second last (i.e. best) 
+#' out-of-bag (OOB) prediction errors stop reducing. 
+#' In the latter case, except for the first iteration, the second last (= best) 
 #' imputed data is returned.
 #' 
+#' OOB prediction errors are quantified as 1 - R^2 for numeric variables, and as 
+#' classification error otherwise. If a variable has been imputed only univariately,
+#' the value is 1.
+#' 
 #' A note on `mtry`: Be careful when passing a non-default `mtry` to 
 #' [ranger::ranger()] because the number of available covariates might be growing during 
 #' the first iteration, depending on the missing pattern. 
@@ -40,16 +47,33 @@
 #'   Furthermore, if `verbose` is positive, the variables used for imputation are 
 #'   listed as well as the variables to be imputed (in the imputation order). 
 #'   This will be useful to detect if some variables are unexpectedly skipped.
-#' @param returnOOB Logical flag. If TRUE, the final average out-of-bag prediction error
-#'   is added to the output as attribute "oob". This does not work in the special case 
-#'   when the variables are imputed univariately.
+#' @param returnOOB Logical flag. If TRUE, the final average out-of-bag prediction 
+#'   errors per variable is added to the resulting data as attribute "oob". 
+#'   Only relevant when `data_only = TRUE` (and when forests are grown).
 #' @param case.weights Vector with non-negative case weights.
+#' @param data_only If `TRUE` (default), only the imputed data is returned.
+#'   Otherwise, a "missRanger" object with additional information is returned.
+#' @param keep_forests Should the random forests of the final imputations
+#'   be returned? The default is `FALSE`. Setting this option will use a lot of memory.
+#'   Only relevant when `data_only = TRUE` (and when forests are grown).
 #' @param ... Arguments passed to [ranger::ranger()]. If the data set is large, 
 #'   better use less trees (e.g. `num.trees = 20`) and/or a low value of 
 #'   `sample.fraction`. The following arguments are incompatible, amongst others: 
 #'   `write.forest`, `probability`, `split.select.weights`, 
 #'   `dependent.variable.name`, and `classification`. 
-#' @returns An imputed `data.frame`.
+#' @returns 
+#'   If `data_only` an imputed `data.frame`. Otherwise, a "missRanger" object with
+#'   the following elements that can be extracted via `$`:
+#'   - `data`: The imputed data.
+#'   - `forests`: When `keep_forests = TRUE`, a list of "ranger" models used to 
+#'     generate the imputed data. `NULL` otherwise.
+#'   - `visit_seq`: Variables to be imputed (in this order).
+#'   - `impute_by`: Variables used for imputation.
+#'   - `best_iter`: Best iteration.
+#'   - `pred_errors`: Per-iteration OOB prediction errors (1 - R^2 for regression,
+#'     classification error otherwise).
+#'   - `mean_pred_errors`: Per-iteration averages of OOB prediction errors.
+#'   
 #' @references
 #'   1. Wright, M. N. & Ziegler, A. (2016). ranger: A Fast Implementation of 
 #'     Random Forests for High Dimensional Data in C++ and R. Journal of Statistical 
@@ -66,32 +90,40 @@
 #' irisImputed <- missRanger(irisWithNA, pmm.k = 3, num.trees = 100)
 #' head(irisImputed)
 #' head(irisWithNA)
+#' 
+#' \dontrun{
+#' # Extended output
+#' imp <- missRanger(irisWithNA, pmm.k = 3, num.trees = 100, data_only = FALSE)
+#' head(imp$data)
+#' imp$pred_errors
+#' 
+#' # If you even want to keep the random forests of the best iteration
+#' imp <- missRanger(
+#'   irisWithNA, pmm.k = 3, num.trees = 100, data_only = FALSE, keep_forests = TRUE
+#' )
+#' imp$forests$Sepal.Width
+#' imp$pred_errors[imp$best_iter, "Sepal.Width"]  # 1 - R-squared
+#' }
 missRanger <- function(data, formula = . ~ ., pmm.k = 0L, maxiter = 10L, 
-                       seed = NULL, verbose = 1, returnOOB = FALSE, 
-                       case.weights = NULL, ...) {
+                       seed = NULL, verbose = 1, returnOOB = FALSE, case.weights = NULL, 
+                       data_only = TRUE, keep_forests = FALSE, ...) {
   if (verbose) {
     cat("\nMissing value imputation by random forests\n")
   }
   
   # 1) INITIAL CHECKS
   bad_args <- c(
-    "write.forest", 
-    "probability", 
-    "split.select.weights",  
-    "dependent.variable.name", 
-    "classification"
+    "write.forest", "probability", "split.select.weights",  
+    "dependent.variable.name", "classification"
   )
   stopifnot(
     "'data' should be a data.frame!" = is.data.frame(data), 
     "'data' should have at least one row and column!" = dim(data) >= 1L, 
     "'formula' should be a formula!" = inherits(formula, "formula"), 
-    length(formula <- as.character(formula)) == 3L,
-    "'pmm.k' must be numeric!" = is.numeric(pmm.k), 
-    "'pmm.k' must be a single number!" = length(pmm.k) == 1L, 
+    "Don't load {formula.tools}. It breaks base R's as.character()" = 
+      length(formula <- as.character(formula)) == 3L,
     "'pmm.k' should not be negative!" = pmm.k >= 0L,
-    "'maxiter' must be numeric!" = is.numeric(maxiter), 
-    "'maxiter' must be a single number!" = length(maxiter) == 1L, 
-    "'maxiter' should not be negative!" = maxiter >= 1L,
+    "'maxiter' should be a positiv number!" = maxiter >= 1L,
     "incompatible ranger arguments" = !(bad_args  %in% names(list(...)))
   )
   if (!is.null(case.weights)) {
@@ -103,8 +135,8 @@ missRanger <- function(data, formula = . ~ ., pmm.k = 0L, maxiter = 10L,
   
   if (!is.null(seed)) {
     set.seed(seed)
-  }  
-  
+  }
+
   # 2) SELECT AND CONVERT VARIABLES TO IMPUTE
   
   # Extract lhs and rhs from formula
@@ -114,47 +146,67 @@ missRanger <- function(data, formula = . ~ ., pmm.k = 0L, maxiter = 10L,
     }
     all.vars(stats::terms.formula(stats::reformulate(z), data = data[1L, ]))
   }
-  relevantVars <- lapply(formula[2:3], parsef)
+  relevant_vars <- lapply(formula[2:3], parsef)
   
   # Pick variables from lhs with some but not all missings
-  toImpute <- relevantVars[[1L]][vapply(data[, relevantVars[[1L]], drop = FALSE], 
-                FUN.VALUE = TRUE, function(z) anyNA(z) && !all(is.na(z)))]
+  pick <- vapply(
+    data[, relevant_vars[[1L]], drop = FALSE], 
+    FUN = function(z) anyNA(z) && !all(is.na(z)),
+    FUN.VALUE = TRUE
+  )
+  to_impute <- relevant_vars[[1L]][pick]
   
   # Try to convert special variables to numeric/factor
   # in order to be safely predicted by ranger
-  converted <- convert(data[, toImpute, drop = FALSE], check = TRUE)
-  data[, toImpute] <- converted$X
+  converted <- convert(data[, to_impute, drop = FALSE], check = TRUE)
+  data[, to_impute] <- converted$X
   
   # Remove variables that cannot be safely converted
-  visitSeq <- setdiff(toImpute, converted$bad)
+  visit_seq <- setdiff(to_impute, converted$bad)
   
   if (verbose) {
     cat("\n  Variables to impute:\t\t")
-    cat(visitSeq, sep = ", ")
+    cat(visit_seq, sep = ", ")
   }
   
-  if (!length(visitSeq)) {
+  if (!length(visit_seq)) {
     if (verbose) {
       cat("\n")
     }
-    return(data)
+    if (data_only) {
+      return(data) 
+    } else {
+      out <- structure(
+        list(
+          data = data,
+          forests = NULL,
+          visit_seq = c(),
+          impute_by = c(),
+          best_iter = 0L,
+          pred_errors = NULL,
+          mean_pred_errors = NULL
+        ), 
+        class = "missRanger"
+      )  
+      return(out)
+    }
   }
   
   # Get missing indicators and order variables by number of missings
-  dataNA <- is.na(data[, visitSeq, drop = FALSE])
-  visitSeq <- names(sort(colSums(dataNA)))
+  data_NA <- is.na(data[, visit_seq, drop = FALSE])
+  visit_seq <- names(sort(colSums(data_NA)))
   
   # 3) SELECT VARIABLES USED TO IMPUTE
   
-  # Variables on the rhs should either appear in "visitSeq" 
+  # Variables on the rhs should either appear in "visit_seq" 
   # or do not contain any missings
-  imputeBy <- relevantVars[[2L]][relevantVars[[2L]] %in% visitSeq | 
-     !vapply(data[, relevantVars[[2L]], drop = FALSE], anyNA, TRUE)]
-  completed <- setdiff(imputeBy, visitSeq)
+  impute_by <- relevant_vars[[2L]][relevant_vars[[2L]] %in% visit_seq | 
+     !vapply(data[, relevant_vars[[2L]], drop = FALSE], anyNA, TRUE)]
+  completed <- setdiff(impute_by, visit_seq)
   
   if (verbose) {
     cat("\n  Variables used to impute:\t")
-    cat(imputeBy, sep = ", ")
+    cat(impute_by, sep = ", ")
     cat("\n")
   }
 
@@ -163,11 +215,15 @@ missRanger <- function(data, formula = . ~ ., pmm.k = 0L, maxiter = 10L,
   # Initialization  
   j <- 1L
   crit <- TRUE
-  verboseDigits <- 4L
-  predError <- stats::setNames(rep(1, length(visitSeq)), visitSeq)
+  dig <- 4L
+  pred_error <- stats::setNames(rep(1, length(visit_seq)), visit_seq)
+  pred_errors <- list()
+  if (keep_forests) {
+    forests <- list()
+  }
   
   if (verbose >= 2) {
-    cat("\n", abbreviate(visitSeq, minlength = verboseDigits + 2L), sep = "\t")
+    cat("\n", abbreviate(visit_seq, minlength = dig + 2L), sep = "\t")
   }
   
   # Looping over iterations and variables to impute
@@ -178,17 +234,20 @@ missRanger <- function(data, formula = . ~ ., pmm.k = 0L, maxiter = 10L,
         cat("\n")
         cat(paste("iter", j))
         cat("\n")
-        pb <- utils::txtProgressBar(0, length(visitSeq), style = 3)
+        pb <- utils::txtProgressBar(0, length(visit_seq), style = 3)
       } else if (verbose >= 2) {
         cat("\niter ", j, ":\t", sep = "")
       }
     }
     
-    dataLast <- data
-    predErrorLast <- predError
-    
-    for (v in visitSeq) {
-      v.na <- dataNA[, v]
+    data_last <- data
+    pred_error_last <- pred_error
+    if (keep_forests) {
+      forests_last <- forests
+    }
+
+    for (v in visit_seq) {
+      v.na <- data_NA[, v]
       
       if (length(completed) == 0L) {
         data[[v]] <- imputeUnivariate(data[[v]])
@@ -200,19 +259,27 @@ missRanger <- function(data, formula = . ~ ., pmm.k = 0L, maxiter = 10L,
           ...
         )
         pred <- stats::predict(fit, data[v.na, completed, drop = FALSE])$predictions
+        
         data[v.na, v] <- if (pmm.k) pmm(
           xtrain = fit$predictions, xtest = pred, ytrain = data[[v]][!v.na], k = pmm.k
         ) else pred
-        predError[[v]] <- fit$prediction.error / (
-          if (fit$treetype == "Regression") stats::var(data[[v]][!v.na]) else 1
-        )
         
-        if (is.nan(predError[[v]])) {
-          predError[[v]] <- 0
+        if (fit$treetype == "Regression") {
+          pred_error[[v]] <- 1 - fit$r.squared
+        } else {  # Classification error
+          pred_error[[v]] <- fit$prediction.error
+        }
+        
+        if (is.nan(pred_error[[v]])) {
+          pred_error[[v]] <- 0
+        }
+
+        if (keep_forests) {
+          forests[[v]] <- fit
         }
       }
       
-      if (j == 1L && (v %in% imputeBy)) {
+      if (j == 1L && (v %in% impute_by)) {
         completed <- union(completed, v)
       }
       
@@ -221,31 +288,54 @@ missRanger <- function(data, formula = . ~ ., pmm.k = 0L, maxiter = 10L,
           utils::setTxtProgressBar(pb, i)
           i <- i + 1L
         } else if (verbose >= 2) {
-          cat(format(round(predError[[v]], verboseDigits), 
-                     nsmall = verboseDigits), "\t")  
+          cat(format(round(pred_error[[v]], dig), nsmall = dig), "\t")  
         }
       }
     }
-
+    
+    pred_errors[[j]] <- pred_error
+    crit <- mean(pred_error) < mean(pred_error_last)
     j <- j + 1L
-    crit <- mean(predError) < mean(predErrorLast)
   }
   
   if (verbose) {
     cat("\n")
   }
   
-  if (j == 2L || (j == maxiter && crit)) {
-    dataLast <- data
-    predErrorLast <- predError
-  }
-  
-  if (returnOOB) {
-    attr(dataLast, "oob") <- predErrorLast 
+  # We take the current iteration if (a) the iteration before did not impute yet
+  # or (b) we had to stop before performance worsened
+  if (j == 2L || (j > maxiter && crit)) {
+    data_last <- data
+    pred_error_last <- pred_error
+    best_iter <- j - 1L
+    if (keep_forests) {
+      forests_last <- forests
+    }
+  } else {
+    best_iter <- j - 2L
   }
   
   # Revert the conversions
-  revert(converted, X = dataLast)
+  data_last <- revert(converted, X = data_last)
+  
+  if (data_only) {
+    if (returnOOB) {
+      attr(data_last, "oob") <- pred_error_last 
+    }
+    return(data_last)
+  }
+  
+  out <- list(
+    data = data_last,
+    forests = if (keep_forests) forests_last,
+    visit_seq = visit_seq,
+    impute_by = impute_by,
+    best_iter = best_iter,
+    pred_errors = do.call(rbind, pred_errors),
+    mean_pred_errors = vapply(pred_errors, FUN = mean, FUN.VALUE = numeric(1))
+  )
+  class(out) <- "missRanger"
+  return(out)
 }
 
 # Helper functions
@@ -286,8 +376,14 @@ convert <- function(X, check = FALSE) {
   stopifnot(is.data.frame(X))
   
   if (!ncol(X)) {
-    return(list(X = X, bad = character(0), vars = character(0), 
-                types = character(0), classes = character(0)))
+    out <- list(
+      X = X, 
+      bad = character(0), 
+      vars = character(0), 
+      types = character(0), 
+      classes = character(0)
+    )
+    return(out)
   }
   
   types <- vapply(X, typeof2, FUN.VALUE = "")
